@@ -41,12 +41,23 @@ class TestSymbols(base.TestCase):
     def test_get_traits_filter_by_prefix(self):
         traits = ot.get_traits('HW_CPU')
         self.assertIn("HW_CPU_X86_SSE42", traits)
+        self.assertIn("HW_CPU_HYPERTHREADING", traits)
         self.assertIn(ot.HW_CPU_X86_AVX2, traits)
         self.assertNotIn(ot.STORAGE_DISK_SSD, traits)
         self.assertNotIn(ot.HW_NIC_SRIOV, traits)
         self.assertNotIn('CUSTOM_NAMESPACE', traits)
         self.assertNotIn('os_traits', traits)
 
+    def test_dunderinit_and_nondunderinit(self):
+        """Make sure we can have both dunderinit'd traits and submodules
+        co-exist in the same namespace.
+        """
+        traits = ot.get_traits('COMPUTE')
+        self.assertIn("COMPUTE_DEVICE_TAGGING", traits)
+        self.assertIn(ot.COMPUTE_DEVICE_TAGGING, traits)
+        self.assertIn("COMPUTE_VOLUME_EXTEND", traits)
+        self.assertIn(ot.COMPUTE_NET_ATTACH_INTERFACE, traits)
+
     def test_get_traits_filter_by_suffix(self):
         traits = ot.get_traits(suffix='SSE42')
         self.assertIn("HW_CPU_X86_SSE42", traits)
@@ -96,3 +107,20 @@ class TestSymbols(base.TestCase):
             match = valid_name.match(t)
             if not match:
                 self.fail("Trait %s does not validate name regex." % t)
+
+    def test_normalize_name(self):
+        values = [
+            ("foo", "CUSTOM_FOO"),
+            ("VCPU", "CUSTOM_VCPU"),
+            ("CUSTOM_BOB", "CUSTOM_CUSTOM_BOB"),
+            ("CUSTM_BOB", "CUSTOM_CUSTM_BOB"),
+            (u"Fu\xdfball", u"CUSTOM_FU_BALL"),
+            ("abc-123", "CUSTOM_ABC_123"),
+            ("Hello, world!  This is a test ^_^",
+             "CUSTOM_HELLO_WORLD_THIS_IS_A_TEST_"),
+            ("  leading and trailing spaces  ",
+             "CUSTOM__LEADING_AND_TRAILING_SPACES_"),
+        ]
+        for test_value, expected in values:
+            result = ot.normalize_name(test_value)
+            self.assertEqual(expected, result)
